@@ -22,6 +22,18 @@
 package org.jboss.as.ejb3.remote;
 
 
+import static org.jboss.as.ejb3.util.MethodInfoHelper.EMPTY_STRING_ARRAY;
+
+import java.io.Serializable;
+import java.lang.reflect.Method;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
+import java.util.HashMap;
+import java.util.Map;
+import java.util.concurrent.ExecutionException;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Future;
+
 import org.jboss.as.ee.component.Component;
 import org.jboss.as.ee.component.ComponentView;
 import org.jboss.as.ee.component.deployers.StartupCountdown;
@@ -57,16 +69,6 @@ import org.wildfly.security.auth.server.SecurityDomain;
 import org.wildfly.security.auth.server.SecurityIdentity;
 import org.wildfly.security.manager.WildFlySecurityManager;
 
-import java.io.Serializable;
-import java.lang.reflect.Method;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
-import java.util.HashMap;
-import java.util.Map;
-import java.util.concurrent.ExecutionException;
-import java.util.concurrent.ExecutorService;
-import java.util.concurrent.Future;
-
 /**
  * {@link EJBReceiver} for local same-VM invocations. This handles all invocations on remote interfaces
  * within the server JVM.
@@ -75,8 +77,6 @@ import java.util.concurrent.Future;
  * @author <a href=mailto:tadamski@redhat.com>Tomasz Adamski</a>
  */
 public class LocalEjbReceiver extends EJBReceiver {
-
-    private static final Object[] EMPTY_OBJECT_ARRAY = {};
     private static final EJBReceiverInvocationContext.ResultProducer.Immediate NULL_RESULT = new EJBReceiverInvocationContext.ResultProducer.Immediate(null);
     private static final AttachmentKey<CancellationFlag> CANCELLATION_FLAG_ATTACHMENT_KEY = new AttachmentKey<>();
 
@@ -115,7 +115,7 @@ public class LocalEjbReceiver extends EJBReceiver {
 
         final Object[] parameters;
         if (invocation.getParameters() == null) {
-            parameters = EMPTY_OBJECT_ARRAY;
+            parameters = EMPTY_STRING_ARRAY;
         } else {
             parameters = new Object[invocation.getParameters().length];
             for (int i = 0; i < parameters.length; ++i) {
@@ -147,7 +147,7 @@ public class LocalEjbReceiver extends EJBReceiver {
                 // entire map of JBoss specific attachments
                 data.put(EJBClientInvocationContext.PRIVATE_ATTACHMENTS_KEY, privateAttachments);
             }
-            // Note: The code here is just for backward compatibility of 1.0.x version of EJB client project
+            // Note: The code here is just for backward compatibility of 1.0.x version of Jakarta Enterprise Beans client project
             // against AS7 7.1.x releases. Discussion here https://github.com/jbossas/jboss-ejb-client/pull/11#issuecomment-6573863
             final boolean txIdAttachmentPresent = privateAttachments.containsKey(AttachmentKeys.TRANSACTION_ID_KEY);
             if (txIdAttachmentPresent) {
@@ -175,10 +175,8 @@ public class LocalEjbReceiver extends EJBReceiver {
                 final boolean isAsync = view.isAsynchronous(method);
                 final boolean oneWay = isAsync && method.getReturnType() == void.class;
                 final boolean isSessionBean = view.getComponent() instanceof SessionBeanComponent;
-                if (isAsync && isSessionBean) {
-                    if (! oneWay) {
-                        interceptorContext.putPrivateData(CancellationFlag.class, flag);
-                    }
+                if (isAsync && isSessionBean && !oneWay) {
+                    interceptorContext.putPrivateData(CancellationFlag.class, flag);
                 }
                 final SecurityContext securityContext;
                 final SecurityDomain securityDomain;
@@ -204,7 +202,7 @@ public class LocalEjbReceiver extends EJBReceiver {
                             result = view.invoke(interceptorContext);
                         } catch (Exception e) {
                             // WFLY-4331 - clone the exception of an async task
-                            receiverContext.resultReady(new CloningExceptionProducer(invocation, resultCloner, e, allowPassByReference));
+                            receiverContext.resultReady(new CloningExceptionProducer(resultCloner, e, allowPassByReference));
                             return;
                         }
                         // if the result is null, there is no cloning needed
@@ -225,7 +223,7 @@ public class LocalEjbReceiver extends EJBReceiver {
                                     intr = true;
                                 } catch (ExecutionException e) {
                                     // WFLY-4331 - clone the exception of an async task
-                                    receiverContext.resultReady(new CloningExceptionProducer(invocation, resultCloner, e, allowPassByReference));
+                                    receiverContext.resultReady(new CloningExceptionProducer(resultCloner, e, allowPassByReference));
                                     return;
                                 }
                             } finally {
@@ -267,11 +265,9 @@ public class LocalEjbReceiver extends EJBReceiver {
             } catch (Exception e) {
                 //we even have to clone the exception type
                 //to make sure it matches
-                receiverContext.resultReady(new CloningExceptionProducer(invocation, resultCloner, e, allowPassByReference));
+                receiverContext.resultReady(new CloningExceptionProducer(resultCloner, e, allowPassByReference));
                 return;
             }
-            //we do not marshal the return type unless we have to, the spec only says we have to
-            //pass parameters by reference
             receiverContext.resultReady(new CloningResultProducer(invocation, resultCloner, result, allowPassByReference));
 
             for(Map.Entry<String, Object> entry : interceptorContext.getContextData().entrySet()) {
@@ -309,20 +305,18 @@ public class LocalEjbReceiver extends EJBReceiver {
     }
 
     static final class CloningExceptionProducer implements EJBReceiverInvocationContext.ResultProducer {
-        private final EJBClientInvocationContext invocation;
         private final ObjectCloner resultCloner;
         private final Exception exception;
         private final boolean allowPassByReference;
 
-        CloningExceptionProducer(final EJBClientInvocationContext invocation, final ObjectCloner resultCloner, final Exception exception, final boolean allowPassByReference) {
-            this.invocation = invocation;
+        CloningExceptionProducer(final ObjectCloner resultCloner, final Exception exception, final boolean allowPassByReference) {
             this.resultCloner = resultCloner;
             this.exception = exception;
             this.allowPassByReference = allowPassByReference;
         }
 
         public Object getResult() throws Exception {
-            throw (Exception) LocalEjbReceiver.clone(resultCloner, exception);
+            throw (Exception) LocalEjbReceiver.clone(Exception.class, resultCloner, exception, allowPassByReference);
         }
 
         public void discardResult() {
@@ -348,6 +342,7 @@ public class LocalEjbReceiver extends EJBReceiver {
         if (!(component instanceof StatefulSessionComponent)) {
             throw EjbLogger.ROOT_LOGGER.notStatefulSessionBean(statelessLocator.getAppName(), statelessLocator.getModuleName(), statelessLocator.getDistinctName(), statelessLocator.getBeanName());
         }
+        component.waitForComponentStart();
         return ((StatefulSessionComponent) component).createSession();
     }
 
@@ -394,7 +389,7 @@ public class LocalEjbReceiver extends EJBReceiver {
         final String distinctName = locator.getDistinctName();
         final String beanName = locator.getBeanName();
         final DeploymentModuleIdentifier moduleIdentifier = new DeploymentModuleIdentifier(appName, moduleName, distinctName);
-        final ModuleDeployment module = deploymentRepository.getValue().getModules().get(moduleIdentifier);
+        final ModuleDeployment module = deploymentRepository.getModules().get(moduleIdentifier);
         if (module == null) {
             throw EjbLogger.ROOT_LOGGER.unknownDeployment(locator);
         }
